@@ -62,6 +62,7 @@
 #include "lib/hikset.h"
 #include "lib/misc.h"
 #include "lib/nid.h"
+#include "lib/str.h"
 #include "lib/stringify.h"
 #include "lib/walloc.h"
 
@@ -540,6 +541,15 @@ pdht_bg_publish(cqueue_t *unused_cq, void *obj)
 	pp->bg->ev = NULL;
 	pp->bg->runs++;
 
+	/*
+	 * If the DHT was disabled dynamically, abort the publishing.
+	 */
+
+	if G_UNLIKELY(!dht_enabled()) {
+		pdht_free_publish(pp, TRUE);
+		return;
+	}
+
 	if (GNET_PROPERTY(publisher_debug) > 1) {
 		g_debug("PDHT starting background %s publish for %s (run #%d)",
 			pdht_type_to_string(pp->type), kuid_to_string(pp->id),
@@ -1322,7 +1332,7 @@ pdht_prox_done(void *u_arg, pdht_error_t code, const pdht_info_t *info)
 				late = "late, ";
 		}
 
-		gm_snprintf(retry, sizeof retry, "%s", compact_time(delay));
+		str_bprintf(retry, sizeof retry, "%s", compact_time(delay));
 
 		g_debug("PDHT PROX %s%spublished to %u node%s%s: %s"
 			" (%stook %s, total %u node%s, proba %.3f%%, retry in %s,"
@@ -1477,6 +1487,9 @@ pdht_prox_publish(bool force)
 	changed = pdht_prox_update_list();
 	publishing = pdht_proxy.proxies_count > 0 && (changed || force);
 
+	if G_UNLIKELY(!dht_enabled())
+		publishing = FALSE;			/* DHT was disabled */
+
 	if (GNET_PROPERTY(publisher_debug) > 1) {
 		g_debug("PDHT PROX list of %u push-prox%s %schanged, %s (%s)",
 			(unsigned) pdht_proxy.proxies_count,
