@@ -59,7 +59,7 @@ enum urpc_ret {
  * @param len			payload length
  * @param arg			user-defined callback parameter
  */
-typedef void (*urpc_cb_t)(enum urpc_ret type, host_addr_t addr, guint16 port,
+typedef void (*urpc_cb_t)(enum urpc_ret type, host_addr_t addr, uint16 port,
 	const void *payload, size_t len, void *arg);
 
 /*
@@ -67,10 +67,10 @@ typedef void (*urpc_cb_t)(enum urpc_ret type, host_addr_t addr, guint16 port,
  */
 
 int urpc_send(const char *what,
-	host_addr_t addr, guint16 port, const void *data, size_t len,
+	host_addr_t addr, uint16 port, const void *data, size_t len,
 	unsigned long timeout, urpc_cb_t cb, void *arg);
 
-gboolean urpc_pending(void);
+bool urpc_pending(void);
 
 void urpc_init(void);
 void urpc_close(void);
