@@ -64,6 +64,7 @@ typedef enum msg_drop_reason {
 	MSG_DROP_TOO_SMALL,
 	MSG_DROP_TOO_LARGE,
 	MSG_DROP_WAY_TOO_LARGE,
+	MSG_DROP_TOO_OLD,
 	MSG_DROP_UNKNOWN_TYPE,
 	MSG_DROP_UNEXPECTED,
 	MSG_DROP_TTL0,
@@ -192,12 +193,19 @@ typedef enum {
 	GNR_GIV_DISCARDED,
 	GNR_QUEUE_CALLBACKS,
 	GNR_QUEUE_DISCARDED,
+	GNR_UDP_READ_AHEAD_COUNT_SUM,
+	GNR_UDP_READ_AHEAD_BYTES_SUM,
+	GNR_UDP_READ_AHEAD_OLD_SUM,
+	GNR_UDP_READ_AHEAD_COUNT_MAX,
+	GNR_UDP_READ_AHEAD_BYTES_MAX,
+	GNR_UDP_READ_AHEAD_DELAY_MAX,
 	GNR_UDP_FW2FW_PUSHES,
 	GNR_UDP_FW2FW_PUSHES_TO_SELF,
 	GNR_UDP_FW2FW_PUSHES_PATCHED,
 	GNR_UDP_UHC_PINGS,
 	GNR_UDP_UHC_PONGS,
 	GNR_UDP_BOGUS_SOURCE_IP,
+	GNR_UDP_RX_TRUNCATED,
 	GNR_UDP_ALIEN_MESSAGE,
 	GNR_UDP_UNPROCESSED_MESSAGE,
 	GNR_UDP_TX_COMPRESSED,
