@@ -234,19 +234,24 @@ eslist_prepend(eslist_t *list, void *data)
 }
 
 static inline void
-eslist_remove_head(eslist_t *list)
+eslist_link_remove_after_internal(eslist_t *list, slink_t *prevlk, slink_t *lk)
 {
-	slink_t *lk;
-
 	g_assert(size_is_positive(list->count));
 	eslist_invariant(list);
 
-	lk = list->head;
 	if G_UNLIKELY(list->tail == lk)
-		list->tail = NULL;
-	list->head = lk->next;
+		list->tail = prevlk;
+
+	if (NULL == prevlk) {
+		/* Removing the head */
+		g_assert(list->head == lk);
+		list->head = lk->next;
+	} else {
+		prevlk->next = lk->next;
+	}
+
 	lk->next = NULL;
-	list->count--;
+	list->count--
 
 	safety_assert(eslist_invariant(list));
 	safety_assert(eslist_length(list->head) == list->count);
@@ -266,12 +271,34 @@ eslist_shift(eslist_t *list)
 		item = NULL;
 	} else {
 		item = ptr_add_offset(list->head, -list->offset);
-		eslist_remove_head(list);
+		eslist_link_remove_after_internal(list, NULL, list->head);
 	}
 
 	return item;
 }
 
+/**
+ * Rotate list by one item to the left.
+ *
+ * The head is inserted back at the tail.
+ */
+void
+eslist_rotate_left(eslist_t *list)
+{
+	slink_t *lk;
+
+	eslist_check(list);
+
+	if G_UNLIKELY(list->count <= 1U)
+		return;
+
+	lk = list->head;
+	eslist_link_remove_after_internal(list, NULL, lk);
+	eslist_link_append_internal(list, lk);
+
+	safety_assert(eslist_invariant(list));
+}
+
 static void
 eslist_link_insert_after_internal(eslist_t *list, slink_t *siblk, slink_t *lk)
 {
@@ -293,15 +320,18 @@ eslist_link_insert_after_internal(eslist_t *list, slink_t *siblk, slink_t *lk)
  * Insert link after another one in list.
  *
  * The sibling must already be part of the list, the new link must not.
+ * If the sibling is NULL, insertion happens at the head of the list.
  */
 void
 eslist_link_insert_after(eslist_t *list, slink_t *sibling_lk, slink_t *lk)
 {
 	eslist_check(list);
-	g_assert(sibling_lk != NULL);
 	g_assert(lk != NULL);
 
-	eslist_link_insert_after_internal(list, sibling_lk, lk);
+	if (NULL == sibling_lk)
+		eslist_link_prepend_internal(list, lk);
+	else
+		eslist_link_insert_after_internal(list, sibling_lk, lk);
 }
 
 /**
@@ -312,16 +342,70 @@ eslist_link_insert_after(eslist_t *list, slink_t *sibling_lk, slink_t *lk)
 void
 eslist_insert_after(eslist_t *list, void *sibling, void *data)
 {
-	slink_t *lk, *siblk;
+	slink_t *lk;
+
+	eslist_check(list);
+	g_assert(data != NULL);
+
+	lk = ptr_add_offset(data, list->offset);
+	if (NULL == sibling) {
+		eslist_link_prepend_internal(list, lk);
+	} else {
+		slink_t *siblk = ptr_add_offset(sibling, list->offset);
+		eslist_link_insert_after_internal(list, siblk, lk);
+	}
+}
+
+/**
+ * Remove data item from list.
+ *
+ * This is usually very inefficient as the list needs to be traversed
+ * to find the previous item.
+ */
+void
+eslist_remove(eslist_t *list, void *data)
+{
+	slink_t *lk, *prevlk, *datalk;
+
+	eslist_check(list);
+	g_assert(data != NULL);
+
+	datalk = ptr_add_offset(data, list->offset);
+	prevlk = NULL;
+
+	for (lk = list->head; lk != NULL; prevlk = lk, lk = lk->next) {
+		if (datalk == lk) {
+			eslist_link_remove_after_internal(list, prevlk, lk);
+			return;
+		}
+	}
+
+	g_assert_not_reached();		/* Item not found in list! */
+}
+
+/**
+ * Remove data item following sibling, if any.
+ *
+ * @return the item removed, NULL if there was nother after sibling.
+ */
+void *
+eslist_remove_after(eslist_t *list, void *sibling)
+{
+	slink_t *lk;
+	void *data;
 
 	eslist_check(list);
 	g_assert(sibling != NULL);
-	g_assert(data != NULL);
 
-	siblk = ptr_add_offset(sibling, list->offset);
-	lk = ptr_add_offset(data, list->offset);
+	lk = ptr_add_offset(sibling, list->offset);
 
-	eslist_link_insert_after_internal(list, siblk, lk);
+	if G_UNLIKELY(NULL == lk->next)
+		return NULL;		/* Nothing after, not an error */
+
+	data = ptr_add_offset(lk->next, -list->offset);
+	eslist_link_remove_after_internal(list, lk, lk->next);
+
+	return data;
 }
 
 /**
@@ -708,7 +792,7 @@ eslist_shuffle(eslist_t *list)
 	 * array.
 	 */
 
-	array = xmalloc(list->count * sizeof array[0]);
+	XMALLOC_ARRAY(array, list->count);
 
 	for (i = 0, lk = list->head; lk != NULL; i++, lk = lk->next) {
 		array[i] = lk;
