@@ -64,9 +64,15 @@ typedef struct assertion_data {
 void G_GNUC_NORETURN NON_NULL_PARAM((1)) /* REGPARM(1) */
 assertion_failure(const assertion_data * const data);
 
+void G_GNUC_NORETURN NON_NULL_PARAM((1)) G_GNUC_PRINTF(2,3)
+assertion_failure_log(const assertion_data * const data, const char *fmt, ...);
+
 void NON_NULL_PARAM((1)) /* REGPARM(1) */
 assertion_warning(const assertion_data * const data);
 
+void NON_NULL_PARAM((1)) G_GNUC_PRINTF(2,3)
+assertion_warning_log(const assertion_data * const data, const char *fmt, ...);
+
 #define RUNTIME_ASSERT(expr) fast_assert(expr, #expr)
 #define RUNTIME_UNREACHABLE() fast_assert_not_reached()
 
@@ -80,6 +86,16 @@ G_STMT_START { \
 	} \
 } G_STMT_END
 
+#define fast_assert_log(expr, expr_string, fmt, ...) \
+G_STMT_START { \
+	if (G_UNLIKELY(!(expr))) { \
+		static const struct assertion_data assertion_data_ = { \
+			_WHERE_, expr_string, __LINE__ \
+		}; \
+		assertion_failure_log(&assertion_data_, (fmt), __VA_ARGS__); \
+	} \
+} G_STMT_END
+
 #define fast_assert_not_reached() \
 G_STMT_START { \
 	static const struct assertion_data assertion_data_ = { \
@@ -90,6 +106,9 @@ G_STMT_START { \
 
 #define return_unless(expr) return_unless_intern((expr), #expr)
 
+#define return_unless_log(expr, fmt, ...) \
+	return_unless_intern_log((expr), #expr, (fmt), __VA_ARGS__)
+
 #define return_unless_intern(expr, expr_string) \
 G_STMT_START { \
 	if (G_UNLIKELY(!(expr))) { \
@@ -101,6 +120,17 @@ G_STMT_START { \
 	} \
 } G_STMT_END
 
+#define return_unless_intern_log(expr, expr_string, fmt, ...) \
+G_STMT_START { \
+	if (G_UNLIKELY(!(expr))) { \
+		static const struct assertion_data assertion_data_ = { \
+			_WHERE_, expr_string, __LINE__ \
+		}; \
+		assertion_warning_log(&assertion_data_, (fmt), __VA_ARGS__); \
+		return; \
+	} \
+} G_STMT_END
+
 #define return_value_unless(expr, val) \
 	return_value_unless_intern((expr), #expr, val)
 
@@ -115,6 +145,17 @@ G_STMT_START { \
 	} \
 } G_STMT_END
 
+#define return_value_unless_intern_log(expr, expr_string, val, fmt, ...) \
+G_STMT_START { \
+	if (G_UNLIKELY(!(expr))) { \
+		static const struct assertion_data assertion_data_ = { \
+			_WHERE_, expr_string, __LINE__ \
+		}; \
+		assertion_warning_log(&assertion_data_, (fmt), __VA_ARGS__); \
+		return (val); \
+	} \
+} G_STMT_END
+
 #define warn_unless(expr, expr_string) \
 G_STMT_START { \
 	if (G_UNLIKELY(!(expr))) { \
@@ -125,8 +166,42 @@ G_STMT_START { \
 	} \
 } G_STMT_END
 
+#define warn_unless_log(expr, expr_string, fmt, ...) \
+G_STMT_START { \
+	if (G_UNLIKELY(!(expr))) { \
+		static const struct assertion_data assertion_data_ = { \
+			_WHERE_, expr_string, __LINE__ \
+		}; \
+		assertion_warning_log(&assertion_data_, (fmt), __VA_ARGS__); \
+	} \
+} G_STMT_END
+
 #define g_soft_assert(expr) warn_unless((expr), #expr)
 
+#define g_soft_assert_log(expr, fmt, ...) \
+	warn_unless_log((expr), #expr, (fmt), __VA_ARGS__)
+
+#define g_assert_log(expr, fmt, ...) \
+	fast_assert_log((expr), #expr, (fmt), __VA_ARGS__)
+
+#define g_return_if_fail_log(expr, fmt, ...) \
+	return_unless_intern_log((expr), #expr, (fmt), __VA_ARGS__)
+
+#define g_return_val_if_fail_log(expr, val, fmt, ...) \
+	return_value_unless_intern_log((expr), #expr, (val), (fmt), __VA_ARGS__)
+
+/* Aliases, for convenience */
+
+#define g_return_unless(expr) return_unless_intern((expr), #expr)
+#define g_return_unless_log(expr, fmt, ...) \
+	return_unless_intern_log((expr), #expr, (fmt), __VA_ARGS__)
+
+#define g_return_val_unless(expr, val) \
+	return_value_unless_intern((expr), #expr, (val))
+
+#define g_return_val_unless_log(expr, val, fmt, ...) \
+	return_value_unless_intern_log((expr), #expr, (val), (fmt), __VA_ARGS__)
+
 #ifdef FAST_ASSERTIONS
 
 #undef g_assert
