@@ -39,11 +39,12 @@
 #include "filename.h"
 #include "ascii.h"
 #include "concat.h"
-#include "glib-missing.h"
-#include "random.h"
-#include "path.h"
-#include "utf8.h"
+#include "glib-missing.h"	/* For g_strlcat() with glib 1.x */
 #include "halloc.h"
+#include "path.h"
+#include "random.h"
+#include "str.h"
+#include "utf8.h"
 
 #include "if/core/guid.h"
 
@@ -446,7 +447,7 @@ filename_unique(const char *path, const char *name, const char *ext,
 	mid_len = utf8_truncate(mid, mid_buf, mid_len + 1);
 	name_len = utf8_truncate(name, name_buf, name_len + 1);
 
-	gm_snprintf(filename_buf, sizeof filename_buf, "%s%s%s",
+	str_bprintf(filename_buf, sizeof filename_buf, "%s%s%s",
 		name_buf, mid_buf, ext_buf);
 
 	pathname = unique_pathname(path, filename_buf, name_is_uniq);
@@ -468,7 +469,7 @@ filename_unique(const char *path, const char *name, const char *ext,
 	name_len = utf8_truncate(name, name_buf, name_len + 1);
 
 	for (i = 0; i < 100; i++) {
-		gm_snprintf(filename_buf, sizeof filename_buf, "%s.%02u%s%s",
+		str_bprintf(filename_buf, sizeof filename_buf, "%s.%02u%s%s",
 			name_buf, i, mid_buf, ext_buf);
 
 		pathname = unique_pathname(path, filename_buf, name_is_uniq);
@@ -487,7 +488,7 @@ filename_unique(const char *path, const char *name, const char *ext,
 	name_len = utf8_truncate(name, name_buf, name_len + 1);
 
 	for (i = 0; i < 100; i++) {
-		gm_snprintf(filename_buf, sizeof filename_buf, "%s.%x%s%s",
+		str_bprintf(filename_buf, sizeof filename_buf, "%s.%x%s%s",
 			name_buf, (unsigned) random_u32(), mid_buf, ext_buf);
 
 		pathname = unique_pathname(path, filename_buf, name_is_uniq);
@@ -511,7 +512,7 @@ filename_unique(const char *path, const char *name, const char *ext,
 		struct guid guid;
 
 		guid_random_fill(&guid);
-		gm_snprintf(filename_buf, sizeof filename_buf, "%s.%s%s%s",
+		str_bprintf(filename_buf, sizeof filename_buf, "%s.%s%s%s",
 			name_buf, guid_hex_str(&guid), mid_buf, ext_buf);
 	}
 
