@@ -36,11 +36,12 @@
 
 #define IS_POWER_OF_2(x) ((x) && 0 == ((x) & ((x) - 1)))
 
-guint32 next_pow2(guint32 n) G_GNUC_CONST;
-int highest_bit_set(guint32 n) G_GNUC_PURE;
-int highest_bit_set64(guint64 n) G_GNUC_PURE;
-int bits_set(guint8 b) G_GNUC_PURE;
-int bits_set32(guint32 v) G_GNUC_PURE;
+uint32 next_pow2(uint32 n) G_GNUC_CONST;
+int highest_bit_set(uint32 n) G_GNUC_PURE;
+int highest_bit_set64(uint64 n) G_GNUC_PURE;
+int bits_set(uint8 b) G_GNUC_PURE;
+int bits_set32(uint32 v) G_GNUC_CONST;
+int ctz64(uint64 n) G_GNUC_CONST;
 
 /**
  * Checks whether the given value is a power of 2.
@@ -48,8 +49,8 @@ int bits_set32(guint32 v) G_GNUC_PURE;
  * @param value a 32-bit integer
  * @return TRUE if ``value'' is a power of 2. Otherwise FALSE.
  */
-static inline G_GNUC_CONST gboolean
-is_pow2(guint32 value)
+static inline ALWAYS_INLINE G_GNUC_CONST bool
+is_pow2(uint32 value)
 #ifdef HAS_BUILTIN_POPCOUNT
 {
 	return 1 == __builtin_popcount(value);
@@ -60,6 +61,64 @@ is_pow2(guint32 value)
 }
 #endif /* HAS_BUILTIN_POPCOUNT */
 
+/**
+ * Populuation count.
+ *
+ * @return number of 1 bits in a 32-bit integer.
+ */
+static inline ALWAYS_INLINE G_GNUC_CONST int
+popcount(uint32 x)
+#ifdef HAS_BUILTIN_POPCOUNT
+{
+	return __builtin_popcount(x);
+}
+#else	/* !HAS_BUILTIN_POPCOUNT */
+{
+	x -= (x >> 1) & 0x55555555;
+	x = ((x >> 2) & 0x33333333) + (x & 0x33333333);
+	x = ((x >> 4) + x) & 0x0f0f0f0f;
+	x += x >> 8;
+	x += x >> 16;
+	return x & 0x1f;	/* At most 32 bits */
+
+}
+#endif	/* HAS_BUILTIN_POPCOUNT */
+
+/**
+ * Count trailing zeroes in a 32-bit integer, -1 for zero.
+ */
+static inline ALWAYS_INLINE G_GNUC_CONST int
+ctz(uint32 x)
+#ifdef HAS_BUILTIN_CTZ
+{
+	return G_UNLIKELY(0 == x) ? -1 : __builtin_ctz(x);
+}
+#else	/* !HAS_BUILTIN_CTZ */
+{
+	return G_UNLIKELY(0 == x) ? 0 : popcount((x & -x) - 1);
+}
+#endif	/* HAS_BUILTIN_CTZ */
+
+/**
+ * Count leading zeroes in a 32-bit integer, 32 for zero.
+ */
+static inline ALWAYS_INLINE G_GNUC_CONST int
+clz(uint32 x)
+#ifdef HAS_BUILTIN_CLZ
+{
+	return G_UNLIKELY(0 == x) ? 32 : __builtin_clz(x);
+}
+#else	/* !HAS_BUILTIN_CLZ */
+{
+	x |= x >> 1;
+	x |= x >> 2;
+	x |= x >> 4;
+	x |= x >> 8;
+	x |= x >> 16;
+	return 32 - popcount(x);
+}
+#endif	/* HAS_BUILTIN_CLZ */
+
 #endif /* _pow2_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
