@@ -42,16 +42,19 @@
 
 void random_init(void);
 uint32 random_value(uint32 max) WARN_UNUSED_RESULT;
-uint64 random_value64(uint64 max) WARN_UNUSED_RESULT;
+uint64 random64_value(uint64 max) WARN_UNUSED_RESULT;
 uint32 random_u32(void) WARN_UNUSED_RESULT;
+uint64 random_u64(void) WARN_UNUSED_RESULT;
 void random_bytes(void *dst, size_t size);
+void random_bytes_with(random_fn_t rf, void *dst, size_t size);
 uint32 random_cpu_noise(void);
 void random_collect(void (*cb)(void));
 void random_pool_append(void *buf, size_t len, void (*cb)(void));
 void random_add(const void *data, size_t datalen);
-double random_double_generate(random_fn_t rf);
-double random_double(void);
-uint32 random_upto(random_fn_t rf, uint32 max);
+double random_double_generate(random_fn_t rf) WARN_UNUSED_RESULT;
+double random_double(void) WARN_UNUSED_RESULT;
+uint32 random_upto(random_fn_t rf, uint32 max) WARN_UNUSED_RESULT;
+uint64 random64_upto(random64_fn_t rf, uint64 max) WARN_UNUSED_RESULT;
 
 #endif /* _random_h_ */
 
