@@ -1,9 +1,4 @@
 #include "common.h"
-#ifdef SDBM
-#include "sdbm.h"
-#else
-#include "ndbm.h"
-#endif
 
 G_GNUC_PRINTF(1, 2) void
 oops(char *fmt, ...)
